package jmssyncreceive;

import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;

import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.log.Log;

/**
 * Message-Driven Bean implementation class for: MyMDB
 */
@MessageDriven(activationConfig = {
	@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge"),
	@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
	@ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/requestQueue"),
})
@Name("myMdb")
public class MyMDB implements MessageListener {
	@Resource(mappedName="java:/JmsXA")
	private ConnectionFactory connFactory;
	@Resource(mappedName="queue/replyQueue")
	private Queue replyQueue;
	@Logger
	private Log log;

	public MyMDB() { }

	private void closeConnection(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (JMSException e) {
				log.error("Error closing JMS connection", e);
				throw new RuntimeException(e);
			}
		}
	}

    public void onMessage(Message message) {
    	log.info("received request");
		Connection conn = null;
		try {
			conn = connFactory.createConnection();
			Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
			Message jmsMsg = session.createObjectMessage("replyMsg");
			MessageProducer producer = session.createProducer(replyQueue);
			producer.send(jmsMsg);
		} catch (JMSException e) {
			log.error("Couldn't send request message", e);
			throw new RuntimeException(e);
		} finally {
			closeConnection(conn);
		}
		log.info("reply sent");
    }
}
