package jmssyncreceive;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;

import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.log.Log;

@Name("webSide")
@Stateless
public class WebSide implements WebSideLocal {
	@Logger
	private Log log;
	@Resource(mappedName="java:/JmsXA")
	private ConnectionFactory connFactory;
	@Resource(mappedName="queue/requestQueue")
	private Queue requestQueue;
	@Resource(mappedName="queue/replyQueue")
	private Queue replyQueue;

	public WebSide() { }

	private void closeConnection(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (JMSException e) {
				log.error("Error closing JMS connection", e);
				throw new RuntimeException(e);
			}
		}
	}

	public void send() {
		Connection conn = null;
		try {
			conn = connFactory.createConnection();
			Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
			Message jmsMsg = session.createObjectMessage("requestMsg");
			MessageProducer producer = session.createProducer(requestQueue);
			producer.send(jmsMsg);
		} catch (JMSException e) {
			log.error("Couldn't send request message", e);
			throw new RuntimeException(e);
		} finally {
			closeConnection(conn);
		}
	}

	public void receive() {
		Connection conn = null;
		try {
			conn = connFactory.createConnection();
			Session session = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);

			// create a consumer with appropriate selector
			log.info("queue value: #0", replyQueue);
			MessageConsumer consumer = session.createConsumer(replyQueue);

			log.info("going to receive");
			Message message = consumer.receive(10000);
			if (message == null) {
				log.info("no reply received, timeout");
				return;
			}
			log.info("message successfully received");
		} catch (JMSException e) {
			log.error("Error receiving reply message", e);
			closeConnection(conn);
			throw new RuntimeException(e);
		} finally {
			closeConnection(conn);
		}
	}
}
